"""
Script to move a subset of GridFS files from one db
(or collection) to another.
"""

import sys
import logging
import argparse
import itertools
import signal

from six.moves import map

import bson
import dateutil.parser
from jaraco.ui import progress
from more_itertools.recipes import consume

from jaraco.mongodb import helper


log = logging.getLogger()


def get_args():
	parser = argparse.ArgumentParser()
	parser.add_argument('source_gfs', type=helper.connect_gridfs)
	parser.add_argument('dest_gfs', type=helper.connect_gridfs)
	parser.add_argument(
		'--include',
		help="a filter of files (regex) to include",
	)
	parser.add_argument(
		'--delete', default=False, action="store_true",
		help="delete files after moving",
	)
	parser.add_argument('--limit', type=int)
	parser.add_argument(
		'--limit-date', type=dateutil.parser.parse,
		help="only move files older than this date",
	)
	return parser.parse_args()


class FileMove:
	include = None
	delete = False
	limit = None
	limit_date = None

	def __init__(self, **params):
		vars(self).update(**params)

	@property
	def filter(self):
		filter = {}
		if self.include:
			filter.update(filename={"$regex": self.include})
		if self.limit_date:
			id_max = bson.objectid.ObjectId.from_datetime(self.limit_date)
			filter.update(_id={"$lt": id_max})
		return filter

	@property
	def source_coll(self):
		return self.source_gfs._GridFS__collection

	@property
	def dest_coll(self):
		return self.dest_gfs._GridFS__collection

	def run(self):
		files = self.source_coll.files.find(
			self.filter,
			batch_size=1,
		)
		limit_files = itertools.islice(files, self.limit)
		count = min(files.count(), self.limit or float('inf'))
		bar = progress.TargetProgressBar(count)
		with SignalTrap(bar.iterate(limit_files)) as items:
			consume(map(self.process, items))

	def process(self, file):
		chunks = self.source_coll.chunks.find(dict(files_id=file['_id']))
		for chunk in chunks:
			self.dest_coll.chunks.insert(chunk)
		self.dest_coll.files.insert(file)
		self.delete and self.source_gfs.delete(file['_id'])


class SignalTrap:
	"""
	A context manager for wrapping an iterable such that it
	is only interrupted between iterations.
	"""

	def __init__(self, iterable):
		self.iterable = iterable

	def __enter__(self):
		self.prev = signal.signal(signal.SIGINT, self.stop)
		return self

	def __exit__(self, *args):
		signal.signal(signal.SIGINT, self.prev)
		del self.prev

	def __iter__(self):
		return self

	def __next__(self):
		return next(self.iterable)
	next = __next__

	def stop(self, signal, frame):
		self.iterable = iter([])


def run():
	logging.basicConfig(stream=sys.stderr, level=logging.INFO)
	args = get_args()

	mover = FileMove(**vars(args))
	mover.run()


if __name__ == '__main__':
	run()



import pytest

from jaraco.mongodb import compat


@pytest.fixture(scope='function')
def database(request, mongodb_instance):
	"""
	Return a clean MongoDB database suitable for testing.
	"""
	db_name = request.node.name
	database = mongodb_instance.get_connection()[db_name]
	yield database
	database.client.drop_database(db_name)


def test_save_no_id(database):
	doc = dict(foo='bar')
	compat.save(database.test_coll, doc)
	assert database.test_coll.find_one()['foo'] == 'bar'


def test_save_new_with_id(database):
	doc = dict(foo='bar', _id=1)
	compat.save(database.test_coll, doc)
	assert database.test_coll.find_one() == doc


def test_save_replace_by_id(database):
	compat.save(database.test_coll, dict(foo='bar', _id=1))

	doc = dict(foo='baz', _id=1)
	compat.save(database.test_coll, doc)
	assert database.test_coll.count() == 1
	assert database.test_coll.find_one() == doc


def test_save_no_id_extant_docs(database):
	"""
	When no id is supplied, a new document should be created.
	"""
	doc = dict(foo='bar')
	compat.save(database.test_coll, dict(doc))
	assert database.test_coll.count() == 1
	compat.save(database.test_coll, doc)
	assert database.test_coll.count() == 2


def test_save_adds_id(database):
	"""
	Ensure _id is added to an inserted document.
	"""
	doc = dict(foo='bar')
	compat.save(database.test_coll, doc)
	assert '_id' in doc



import functools

from jaraco.collections import Projection


def save(coll, to_save):
	"""
	Pymongo has deprecated the save logic, even though
	MongoDB still advertizes that logic in the core API:
	https://docs.mongodb.com/manual/reference/method/db.collection.save/

	This function provides a compatible interface.
	"""
	filter = Projection(['_id'], to_save)
	upsert_replace = functools.partial(coll.replace_one, filter, upsert=True)
	op = upsert_replace if filter else coll.insert_one
	return op(to_save)


def query_or_command(op):
	"""
	Given an operation from currentOp, return the query or command
	field as it changed in MongoDB 3.2 for indexing operations:

	https://docs.mongodb.com/manual/reference/method/db.currentOp/#active-indexing-operations
	"""
	return op.get('command') or op.get('query')
